/***********************************************************************/
/*                                                                     */
/*   SVMTorch                                                          */
/*                                                                     */
/*   Author: Ronan Collobert                                           */
/*   Date: 21.07.2000                                                  */
/*                                                                     */
/*   Copyright (c) 2000  IDIAP - All rights reserved                   */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/
  

#include "general.h"
#include "chunker.h"
#include "kernel.h"
#include "solveurs.h"
#include "svm.h"

char *file_input;
char *file_output;
int mode_input;
int mode_output;
char *version;

void help(void)
{
	printf("***********************************************************************************\n");
	printf("* The Trebolloc Compagnie presents :\n");
	printf("* %s\n", version);
	printf("*\n");
	printf("*\n");
	printf("* Usage : svm_torch [options] example_file model_file\n");
	printf("* [In this public version the chunk size != 2 is not implemented]\n");
	printf("*\n");
	printf("*\n");
	printf("* Arguments :\n");
	printf("*        example_file  -> file with training data\n");
	printf("*        model_file    -> file to store the model\n");
	printf("* General options :\n");
	printf("*        -help         -> this little help\n");
	printf("*        -v            -> print some messages\n");
	printf("* Learning options :\n");
	printf("*        -rm           -> regression mode\n");
	printf("*        -c  float     -> C : trade-off between training error\n");
	printf("*                         and margin [%f]\n", C);
	printf("*        -eps          -> The width of the error pipe in regression mode [%f]\n", epsilon);
	printf("* Kernel options :\n");
	printf("*        -t  int       -> type of kernel functions :\n");
	printf("*                          0 -> linear\n");
	printf("*                          1 -> polynomial (s a*b+c)^d\n");
	printf("*                          2 -> radial basis function exp(-|a-b|^2 / (std*std)) [default]\n");
	printf("*                          3 -> sigmoid tanh(s a*b + c)\n");
	printf("*                          4 -> user defined kernel [see kernel_user.c]\n");
	printf("*        -d  int       -> parameter d in polynomial kernel\n");
	printf("*        -st float     -> parameter std in rbf kernel\n");
	printf("*        -s  float     -> parameter s in sigmoid/poly kernel\n");
	printf("*        -r  float     -> parameter c in sigmoid/poly kernel\n");
	printf("*        -u  string    -> parameter of user defined kernel\n");
	printf("* Optimization options :\n");
	printf("*        -q  int       -> size of QP-subproblems [%d]\n", chunk_size);
	printf("*        -m  int >= 1  -> size of cache in megabyte [%d Mo]\n", cache_size/1048576);
	printf("*        -e  float     -> allow that error for termination criterion [%.1e]\n", precision);
	printf("*        -h  int       -> minimal number of iterations a variable needs\n");
	printf("*                         to be shrinked [%d]\n", iter_min_to_shrink);
	printf("* Nightmare options :\n");
	printf("*        -xe float > 0 -> sometimes used inside as precision [%.1e]\n", eps);
	printf("*        -xp int       -> if there is at least (number of examples)/(xp) variables\n");
	printf("*                         to shrink, we shrink. Else not. [%d]\n", proportion_to_shrink);
	printf("*        -xi float     -> precision of the internal optimizer [%.1e]\n", i_precision);
	printf("*                         that's not for the analytic one !\n");
	printf("* Input options :\n");
	printf("*        -ae           -> ASCII examples\n");
	printf("* Output options :\n");
	printf("*        -am           -> ASCII model\n");
	printf("*\n");
	printf("*********>> SVMTorch is (c) Ronan Collobert 2000 (IDIAP) <<***********************\n\n");

	exit(0);
}

void scan_cmd(int argc, char **argv)
{
	int i = 1;
	int erreur;
	int maxa;

	if(argc < 3)
		help();

	maxa = argc-2;
	while(i < maxa)
	{
		erreur = 1;

		if(!strcmp(argv[i], "-help"))
			help();

		if(!strcmp(argv[i], "-v"))
		{
			debug = true;
			erreur = 0;
		}

		if(!strcmp(argv[i], "-ae"))
		{
			mode_input = 1;
			erreur = 0;
		}

		if(!strcmp(argv[i], "-am"))
		{
			mode_output = 1;
			erreur = 0;
		}

		if(!strcmp(argv[i], "-rm"))
		{
			regression = 1;
			erreur = 0;
		}

		if(!strcmp(argv[i], "-c"))
		{
			i++;
			if(i < maxa)
				C = atof(argv[i]);
			else
				help();

			erreur = 0;
		}		

		if(!strcmp(argv[i], "-t"))
		{
			i++;
			if(i < maxa)
			{
				t_kernel = atoi(argv[i]);
				
				switch(t_kernel)
				{
				 case 0 :
					 kernel = kernel_linear;
					 break;
				 case 1 :
					 kernel = kernel_poly;
					 break;
				 case 2 :
					 kernel = kernel_exp;
					 break;
				 case 3 :
					 kernel = kernel_sigm;
					 break;
				 case 4 :
					 kernel = kernel_user;
					 break;
				 default :
					 help();
					 
				}
			}
			else
				help();

			erreur = 0;
		}		

		if(!strcmp(argv[i], "-d"))
		{
			i++;
			if(i < maxa)
				k_poly = atoi(argv[i]);
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-st"))
		{
			i++;
			if(i < maxa)
			{
				k_exp = atof(argv[i]);
				k_exp = 1.0 /(k_exp*k_exp);
			}			
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-s"))
		{
			i++;
			if(i < maxa)
				k_sigm_s = atof(argv[i]);
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-r"))
		{
			i++;
			if(i < maxa)
				k_sigm_c = atof(argv[i]);
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-u"))
		{
			i++;
			if(i < maxa)
			{
				k_user = argv[i];
				initKernelUser();
			}
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-q"))
		{
			i++;
			if(i < maxa)
				setChunkSize(atoi(argv[i]));
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-m"))
		{
			i++;
			if(i < maxa)
				setCacheSize(atoi(argv[i]));
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-e"))
		{
			i++;
			if(i < maxa)
				precision = atof(argv[i]);
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-h"))
		{
			i++;
			if(i < maxa)
				setIterMinToShrink(atoi(argv[i]));
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-xe"))
		{
			i++;
			if(i < maxa)
				setEps(atof(argv[i]));
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-eps"))
		{
			i++;
			if(i < maxa)
				epsilon = atof(argv[i]);
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-xp"))
		{
			i++;
			if(i < maxa)
				setProportionToShrink(atoi(argv[i]));
			else
				help();

			erreur = 0;
		}	

		if(!strcmp(argv[i], "-xi"))
		{
			i++;
			if(i < maxa)
				setIPrecision(atof(argv[i]));
			else
				help();

			erreur = 0;
		}

		i++;

		if(erreur)
			help();
	}

	file_input = argv[argc-2];
	file_output = argv[argc-1];
}

int main(int argc, char **argv)
{
	SVM *svm;

	/* init diverses */
	version = (char *)malloc(sizeof(char)*80);
	strcpy(version, "SVMTorch ");
	strcat(version, VERSION);
	mode_input = 0;
	mode_output = 0;
	regression = 0;

	/* Global Kernel */
	kernelInit();

	/* Global chunker */
	chunkerInit();

	/* Scan Command Line */
	scan_cmd(argc, argv);
	
	if(chunk_size == 2)
	{
		if(regression)
			launchSubProblem = RanalyticSolve;
		else
			launchSubProblem =  analyticSolve;

		i_opti_is_set = true;
	}
	else
	{
		printf("Chunk != 2 not yet implemented\n");
		exit(0);
	}


	/******************************** START ****************************/

	svm = initSVM();

	printf("\nLoading...\n");
	if(mode_input)
		createFileAsciiSVM(svm, file_input);
	else
		createFileBinSVM(svm, file_input);

	checkSVM(svm);
	learnSVM(svm);
	svSVM(svm);

	if(mode_output)
		saveFileAsciiSVM(svm, file_output);
	else
		saveFileBinSVM(svm, file_output);
	
	destroySVM(svm);

	return(0);
}
